----------------------------------------------------------------------
----                                                              ----
---- ATARI Falcon COMBEL compatible IP Core	    	              ----
----                                                              ----
---- This file is part of the SUSKA ATARI clone project.          ----
---- http://www.experiment-s.de                                   ----
----                                                              ----
---- Description:                                                 ----
---- Atari's COMBEL with all features to reach                    ----
---- ATARI Falcon compatibility.                                  ----
----                                                              ----
---- This is the package file containing the component            ----
---- declarations.                                                ----
----                                                              ----
----                                                              ----
---- To Do:                                                       ----
---- -                                                            ----
----                                                              ----
---- Author(s):                                                   ----
---- - Wolfgang Foerster, wf@experiment-s.de; wf@inventronik.de   ----
----                                                              ----
----------------------------------------------------------------------
----                                                              ----
---- Copyright � 2009-2010 Wolfgang Foerster Inventronik GmbH.    ----
---- All rights reserved. No portion of this sourcecode may be    ----
---- reproduced or transmitted in any form by any means, whether  ----
---- by electronic, mechanical, photocopying, recording or        ----
---- otherwise, without my written permission.                    ----
----                                                              ----
----------------------------------------------------------------------
-- 
-- Revision History
-- 
-- Revision 2K9B  2009/12/24 WF
--   Initial Release.
-- 

library work;
use work.COMBEL_PKG.all;

library ieee;
use ieee.std_logic_1164.all;

package COMBEL_PKG is
    type BANKTYPE is (K128, K512, K2048, K8192, K256x16, K512x16, K1024x16, K2048x16);
    type MCU_PHASE_TYPE is (IDLE, RAM, VIDEO, REFRESH);
    -- Component declarations:
    component CLOCKS
    port (
        CLK	    : in bit;

        CLK_04	: out bit;
        CLK_08	: out bit;
        CLK_064	: out bit
    );
    end component;

    component BLITTER_TOP
        port (
            CLK			: in bit;
            RESET 		: in bit;
            AS_INn		: in bit;
            AS_OUTn		: out bit;
            LDS_INn		: in bit;
            LDS_OUTn	: out bit;
            UDS_INn		: in bit;
            UDS_OUTn	: out bit;
            RWn_IN		: in bit;
            RWn_OUT		: out bit;
            DTACK_INn	: in bit;
            DTACK_OUTn	: out bit;
            BERRn		: in bit;
            BMODE       : in bit;
            FC_IN		: in bit_vector(2 downto 0);
            FC_OUT		: out std_logic_vector(2 downto 0);
            BUSCTRL_EN	: out bit;
            INTn		: out bit;
            ADR_IN		: in bit_vector(31 downto 1);
            ADR_OUT		: out std_logic_vector(31 downto 1);
            ADR_EN		: out bit;
            DATA_IN		: in std_logic_vector(15 downto 0);
            DATA_OUT	: out std_logic_vector(15 downto 0);
            DATA_EN		: out bit;
            BGIn		: in bit;
            BRn			: out bit;
            BGACK_INn	: in bit;
            BGACK_OUTn	: out bit;
            BGOn		: out bit
        );
    end component;

    component ADRDEC
        port (  
            ADR				        : in bit_vector(31 downto 1);
            RWn				        : in bit;
            ROMSEL_FC_E0n           : in bit;
            LDSn			        : in bit;
            UDSn			        : in bit;
            ASn				        : in bit;
            VPAn			        : out bit;
            VMAn			        : in bit;
            FC				        : in bit_vector(2 downto 0);
            ROM_0n			        : out bit;
            ROM_1n			        : out bit;
            ROM_2n			        : out bit;
            ROM_3n			        : out bit;
            ROM_4n			        : out bit;
            ROM_5n			        : out bit;
            ROM_6n			        : out bit;
            ACIACS			        : out bit;
            MFPCSn			        : out bit;
            SNDCSn			        : out bit;
            SCCn			        : out bit;
            SCCAB			        : out bit;
            RTCCSn			        : out bit;
            JOY_RS			        : out bit;
            PAD0X_RS		        : out bit;
            PAD0Y_RS		        : out bit;
            PAD1X_RS		        : out bit;
            PAD1Y_RS		        : out bit;
            BUTTON_RS		        : out bit;
            R8006CS		            : out bit;
            R8007CS		            : out bit;
            FPUCS		            : out bit;
            VCS		                : out bit;
            MEM_CONFIG_RS	        : out bit;
            LINE_OFFS_RS		    : out bit;
            LINE_WIDTH_RS		    : out bit;
            VIDEO_BASE_HIWORD_RS	: out bit;
            VIDEO_BASE_LOWORD_RS	: out bit;
            VIDEO_COUNT_HIWORD_RS	: out bit;
            VIDEO_COUNT_LOWORD_RS	: out bit;
            VIDEO_BASE_HI_RS	    : out bit;
            VIDEO_BASE_MID_RS	    : out bit;
            VIDEO_BASE_LOW_RS	    : out bit;
            VIDEO_COUNT_HI_RS	    : out bit;
            VIDEO_COUNT_MID_RS	    : out bit;
            VIDEO_COUNT_LOW_RS	    : out bit;
            SHMOD_ST_SHADOW_RS      : out bit;
            VMODE_SHADOW_RS         : out bit;
            RAMn			        : out bit
          );
    end component;

    component MCU_TOP
        port(
            CLK				        : in bit;
            RESET   		        : in bit;
            LDSn, UDSn		        : in bit;
            RWn				        : in bit;
            ADR_IN		            : in bit_vector(31 downto 1);
            RAMn			        : in bit;
            RAMH			        : out bit;
            VREQ                    : in bit;
            EVENn_ODD               : in bit;
            RDAT			        : out bit;
            WDAT			        : out bit;
            VLD 			        : out bit;
            VINTn			        : in bit;
            HINTn			        : in bit;
            VIDEO_BASE_HIWORD_RS	: in bit;
            VIDEO_BASE_LOWORD_RS	: in bit;
            VIDEO_COUNT_HIWORD_RS	: in bit;
            VIDEO_COUNT_LOWORD_RS	: in bit;
            VIDEO_BASE_HI_RS	    : in bit;
            VIDEO_BASE_MID_RS	    : in bit;
            VIDEO_BASE_LOW_RS	    : in bit;
            VIDEO_COUNT_HI_RS	    : in bit;
            VIDEO_COUNT_MID_RS	    : in bit;
            VIDEO_COUNT_LOW_RS	    : in bit;
            SHMOD_ST_SHADOW_RS      : in bit;
            VMODE_SHADOW_RS         : in bit;
            MEM_CONFIG_RS		    : in bit;
            LINE_OFFS_RS		    : in bit;
            LINE_WIDTH_RS		    : in bit;
            DTACKn			        : out bit;
            DATA_IN			        : in std_logic_vector(15 downto 0);
            DATA_OUT		        : out std_logic_vector(15 downto 0);
            DATA_EN			        : out bit;
            BANK0_TYPE              : out BANKTYPE;
            RAS1n			        : out bit;
            CAS1Hn  		        : out bit;
            CAS1Ln  		        : out bit;
            RAS0n			        : out bit;
            CAS0Hn			        : out bit;
            CAS0Ln  		        : out bit;
            WEn				        : out bit;
            RAM_MADR			    : out bit_vector(26 downto 1)
        );
    end component;

    component MCU_CTRL
        port (  
            CLK				: in bit;
            RESET 			: in bit;
            LDSn, UDSn, RWn	: in bit;
            M_ADR			: in bit_vector(31 downto 1);
            RAMn			: in bit;
            RAMH			: out bit;
            VREQ            : in bit; 
            MEM_CONFIG_RS	: in bit;
            MCU_PHASE		: out MCU_PHASE_TYPE;
            BANK0_TYPE      : out BANKTYPE;
            VINTn			: in bit;
            RAS0n			: out bit;
            CAS0Hn			: out bit;
            CAS0Ln			: out bit;
            WEn				: out bit;
            RAS1n			: out bit;
            CAS1Hn			: out bit;
            CAS1Ln			: out bit;
            RDAT 			: out bit;
            WDAT 			: out bit;
            VLD 			: out bit;
            REF_EN			: out bit;
            VIDEO_CNT_EN	: out bit;
            VIDEO_CNT_LOAD	: out bit;
            DTACKn			: out bit;
            DATA_IN			: in std_logic_vector(7 downto 0);
            DATA_OUT		: out bit_vector(7 downto 0);
            DATA_EN			: out bit
        );
    end component;

    component MCU_VIDEO_COUNTER
    port (  CLK		                : in bit;
            RESET 	                : in bit;
            RWn		                : in bit;
            VIDEO_BASE_HIWORD_RS	: in bit;
            VIDEO_BASE_LOWORD_RS	: in bit;
            VIDEO_COUNT_HIWORD_RS	: in bit;
            VIDEO_COUNT_LOWORD_RS	: in bit;
            VIDEO_BASE_HI_RS	    : in bit;
            VIDEO_BASE_MID_RS	    : in bit;
            VIDEO_BASE_LOW_RS	    : in bit;
            VIDEO_COUNT_HI_RS	    : in bit;
            VIDEO_COUNT_MID_RS	    : in bit;
            VIDEO_COUNT_LOW_RS	    : in bit;
            SHMOD_ST_SHADOW_RS      : in bit;
            VMODE_SHADOW_RS         : in bit;
            VINTn			        : in bit;
            HINTn			        : in bit;
            EVENn_ODD			    : in bit;
            VIDEO_COUNT_EN		    : in bit;
            VIDEO_COUNT_LOAD	    : in bit;
            LINE_OFFS_RS		    : in bit;
            LINE_WIDTH_RS		    : in bit;
            VIDEO_ADR_OUT		    : out bit_vector(31 downto 1);
            DATA_IN			        : in std_logic_vector(15 downto 0);
            DATA_OUT		        : out std_logic_vector(15 downto 0);
            DATA_EN			        : out bit
          );
    end component;

    component INTERRUPTS
        port(
            RESET 			: in bit;
            CLK				: in bit;
            ADR_HI			: in bit_vector(19 downto 16);
            ADR_LO			: in bit_vector(3 downto 1);
            FC				: in bit_vector(2 downto 0);
            ASn				: in bit;
            EINT1n			: in bit;
            EINT3n			: in bit;
            EINT5n			: in bit;
            EINT7n			: in bit;
            MFPINTn			: in bit;
            HINTn			: in bit;
            VINTn			: in bit;
            AVECn			: out bit;
            IACKn			: out bit;
            SCCIACKn		: out bit;
            IPLn			: out bit_vector(2 downto 0) -- STE GLUE.
        );
    end component;

    component ERRHANDLER
        port(
            RESET 			: in bit;
            CLK				: in bit;
            ASn				: in bit;

            BERRn			: out bit
        );
    end component;

    component JOYPORT_PADDLES
        port(
            CLK             : in bit;
            KHz_500	        : in bit;
            RESET 			: in bit;
            RWn				: in bit;
            DATA_IN			: in std_logic_vector(1 downto 0);
            DATA_OUT		: out std_logic_vector(15 downto 0);
            DATA_EN			: out bit;
            PAD0X_RS		: in bit;
            PAD0Y_RS		: in bit;
            PAD1X_RS		: in bit;
            PAD1Y_RS		: in bit;
            PAD0X_INHn		: in bit;
            PAD0Y_INHn		: in bit;
            PAD1X_INHn		: in bit;
            PAD1Y_INHn		: in bit;
            PADRSTn			: out bit
        );
    end component;

    component WF_IDE
        port (
            RESET 			: in bit;

            ADR				: in bit_vector(31 downto 4);

            ASn				: in bit;
            LDSn            : in bit;
            RWn				: in bit;
            DTACKn			: out bit;
            HDINTn		: out bit;
            IDE_INTRQ		: in bit;
            IDE_IORDY		: in bit;
            -- PDIAG		: in bit;
            -- DASP			: in bit;
            -- DMARQ		: in bit;
            -- DMACKn		: out bit;
            IDE_RESn		: out bit;
            CS0n			: out bit;
            CS1n			: out bit;
            IORDn			: out bit;
            IOWRn			: out bit;
            IDE_D_EN_INn	: out bit;
            IDE_D_EN_OUTn	: out bit
          );
    end component;
end COMBEL_PKG;
