----------------------------------------------------------------------
----                                                              ----
---- This file is part of the SUSKA ATARI clone project.          ----
---- http://www.experiment-s.de                                   ----
----                                                              ----
---- Description:                                                 ----
----                                                              ----
----                                                              ----
----                                                              ----
---- To Do:                                                       ----
---- -                                                            ----
----                                                              ----
---- Author(s):                                                   ----
---- - Wolfgang Foerster, wf@experiment-s.de; wf@inventronik.de   ----
----                                                              ----
----------------------------------------------------------------------
----                                                              ----
---- Copyright  2009-2010 Wolfgang Foerster Inventronik GmbH.    ----
---- All rights reserved. No portion of this sourcecode may be    ----
---- reproduced or transmitted in any form by any means, whether  ----
---- by electronic, mechanical, photocopying, recording or        ----
---- otherwise, without my written permission.                    ----
----                                                              ----
----------------------------------------------------------------------
-- 
-- Revision History
-- 
-- Revision 2K9B 2009/12/24 WF
--   Initial reease
-- Revision 2K13B 20131224 WF
--   
-- 

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

library ieee;
use ieee.std_logic_1164.all;

entity WF_SD_DRIVES is
    port(
        RESETn              : in bit;
        CLK                 : in bit;

        CD_IN               : in std_logic_vector(7 downto 0);
        CD_OUT              : out std_logic_vector(7 downto 0);
        CD_EN               : out bit;
        CR_Wn		        : in bit;
        CA1		            : in bit;

        HDCSn			    : in bit;
        HDRQ				: out bit;
        ACKn				: in bit;
        HDINTn              : out bit;

        FDD_RDn             : out bit;
        FDD_TR00n           : out bit;
        FDD_IPn		        : out bit;
        FDD_WPRTn	        : out bit;
        FDD_FDTYPE          : out bit;
        FDD_MO              : in bit;
        FDD_WG              : in bit;
        FDD_WD              : in bit;
        FDD_STEP            : in bit;
        FDD_DIRC            : in bit;
        
        JOY_DIS             : out bit;
        LED_1               : out bit;
        LED_2               : out bit;

        SDC_EN              : out bit;
        SDC_MISO            : in bit;
        SDC_MOSI            : out bit;
        SDC_CSn             : out bit;
        SDC_SCK             : out bit;
        SDC_CDn             : in bit;
        SDC_WP              : in bit;
        SDC_PWRn            : out bit
    );
end WF_SD_DRIVES;

architecture BEHAVIOUR of WF_SD_DRIVES is
component ACSI_SDHD_TOP_SOC
	port (
		CLK_SD          : in bit;
		CLK_BUS         : in bit;
		RESETn	        : in bit;
		DATA_IN		    : in bit_vector(7 downto 0);
		DATA_OUT	    : out bit_vector(7 downto 0);
		DATA_EN		    : out bit;
		CR_Wn     	    : in bit;
		CA1             : in bit;
		CSn	            : in bit;
		ACKn	        : in bit;
		REQn	        : out bit;
		INTn            : out bit;
        TARGET_IDn      : in bit_vector(2 downto 0);
        SDC_EN          : out bit;
        SDC_CDn         : in bit;
        SDC_WP          : in bit;
        SDC_MISO        : in bit;
        SDC_CSn         : out bit;
        SDC_MOSI        : out bit;
        SDC_SCK         : out bit;
        SDC_PWRn        : out bit;
        SDSC1           : out bit;
        SDSC2           : out bit;
        SDHCXC          : out bit;
        SPEED_MODE      : out bit_vector(1 downto 0);
        ERROR           : out bit
	);
end component;
signal ERROR            : bit;
signal REQn             : bit;
signal CD_EN_SDHD       : bit;
signal CD_OUT_SDHD      : bit_vector(7 downto 0);
signal SDHCXC           : bit;
signal SDSC1            : bit;
signal SDSC2            : bit;
signal SPEED_MODE       : bit_vector(1 downto 0);
begin
    -- Inactive.
    FDD_RDn <= '1';
    FDD_TR00n <= '1';
    FDD_IPn <= '1';
    FDD_WPRTn <= '1';
    FDD_FDTYPE <= '1';

    P_LED: process
    -- This logic controlls the two LED. Where the second LED2 shows bus
    -- traffic from or to the SD card disk, the LED1 has several functions.
    -- The first 5 seconds after a system reset, the LED1 indicates the type
    -- of the inserted SD card. See the information in the header of this
    -- section. After 5s the LED1 is the operation indicator with low
    -- illuminance. If an error occurs, the LED1 and LED2 alternate with a
    -- frequency of about 2Hz.
    variable TMP_1  : std_logic_vector(25 downto 0);
    variable TMP_2  : integer range 0 to 7;
    begin
        wait until CLK = '1' and CLK' event;
        if RESETn = '0' then
            TMP_1 := (others => '0');
            TMP_2 := 0;
        elsif SDC_CDn = '1' then
            TMP_2 := 0;
            TMP_1 := TMP_1 + '1';
        else
            TMP_1 := TMP_1 + '1';
            if TMP_1(25 downto 24) = "00" and TMP_1(23 downto 0) = x"000000" and TMP_2 < 7 then
                TMP_2 := TMP_2 + 1;
            end if;
        end if;
        
        if ERROR = '1' then
            LED_1 <= To_Bit(TMP_1(24));
        elsif SDC_CDn = '1' or TMP_2 > 5 then
            if TMP_1(7 downto 0) < x"50" then
                LED_1 <= '1';
            else
                LED_1 <= '0';
            end if;
        elsif SDSC1 = '1' then
            LED_1 <= To_Bit(TMP_1(25));
        elsif SDSC2 = '1' then
            LED_1 <= To_Bit(TMP_1(24));
        elsif SDHCXC = '1' then
            LED_1 <= To_Bit(TMP_1(23));
        else
            LED_1 <= '0';
        end if;
        
        if ERROR = '1' then
            LED_2 <= not To_Bit(TMP_1(24));
        elsif REQn = '0' then
            LED_2 <= '1';
        elsif SDC_CDn = '1' then
            LED_2 <= '0'; -- No Card.
        elsif SPEED_MODE > "01" then
            LED_2 <= '0';
        else
            if TMP_1(7 downto 0) < x"18" then
                LED_2 <= '1';
            else
                LED_2 <= '0';
            end if;
        end if;
    end process P_LED;

    JOY_DIS <= '0'; -- We need not to disable the Joystick when the SD-Card is driven in SPI mode.

    CD_OUT <= To_StdLogicVector(CD_OUT_SDHD) when CD_EN_SDHD = '1' else x"00";
    CD_EN <= CD_EN_SDHD;

    HDRQ <= not REQn;

    I_ACSI_SDHD: ACSI_SDHD_TOP_SOC
        port map(
            CLK_SD          => CLK,
            CLK_BUS         => CLK,
            RESETn	        => RESETn,
            DATA_IN		    => To_BitVector(CD_IN),
            DATA_OUT	    => CD_OUT_SDHD,
            DATA_EN		    => CD_EN_SDHD,
            CR_Wn     	    => CR_Wn,
            CA1             => CA1,
            CSn	            => HDCSn,
            ACKn	        => ACKn,
            REQn	        => REQn,
            INTn            => HDINTn,
            TARGET_IDn      => "000",
            SDC_EN          => SDC_EN,
            SDC_CDn         => SDC_CDn,
            SDC_WP          => SDC_WP,
            SDC_MISO        => SDC_MISO,
            SDC_CSn         => SDC_CSn,
            SDC_MOSI        => SDC_MOSI,
            SDC_SCK         => SDC_SCK,
            SDC_PWRn        => SDC_PWRn,
            SDSC1           => SDSC1,
            SDSC2           => SDSC2,
            SDHCXC          => SDHCXC,
            SPEED_MODE      => SPEED_MODE,
            ERROR           => ERROR
        );
end BEHAVIOUR;
